/*
  mtoolsrcgen - a utility to generate .mtoolsrc files
  for Atari hard disk images.

  Copyright (C) 2002  Standa Opichal <opichals@seznam.cz>

  Inspired by Sven de Marothy's mtoolutil tool.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Version: 0.9.6
 */

#include <stdio.h>
#include <netinet/in.h>

#include "types.h"
#include "kernelfake.h"

#include "check.h"
#include "atari.h"


static const int SECTOR_SIZE = 512;
static int doSwap = 0;
static char* imageFileName = NULL;


void usage();


unsigned char *read_dev_sector(struct block_device *blk_dev, unsigned long pos, Sector *sect )
{
	fseek( blk_dev->fh, pos * SECTOR_SIZE, 0 );
	fread( sect, SECTOR_SIZE, 1, blk_dev->fh );
	if ( doSwap ) {
		char *buf = (char*)sect;
		int i;
		for (i=0; i<SECTOR_SIZE; i+=2) {
			char temp = buf[i];
			buf[i] = buf[i+1];
			buf[i+1] = temp;
		}
	}
	return (char*)sect;
}

void add_gd_partition(struct gendisk *hd, int minor, int start, int size)
{
	printf("### PAT: offset=%08x from=%08x size=%08x\n", start*SECTOR_SIZE, start, size );
	printf("drive %c:\n ", ('b'+minor));
	if ( doSwap )
		printf(" swap" );
	printf(" offset=%u file=\"%s\"\n\n", start*SECTOR_SIZE, imageFileName );
}

int mtool_atari_partition (struct gendisk *hd, struct block_device *bdev,
						   unsigned long first_sector, int first_part_minor)
{
	// get the nr_sects from the file
	fseek(bdev->fh, 0x1C2L, 0);
	if (fread(&hd->part[0].nr_sects, 4, 1, bdev->fh) == 1) {
		if ( doSwap )
			hd->part[0].nr_sects = ( hd->part[0].nr_sects >> 16 ) || (( hd->part[0].nr_sects & 0xffff ) << 16 );
		hd->part[0].nr_sects = be32_to_cpu(hd->part[0].nr_sects);

		return atari_partition( hd, bdev, first_sector, first_part_minor );
	}

	return -1;
}

int main(int argc, char *argv[]){

	struct block_device blk_dev;
	struct gendisk hd;
	int result;

	hd.max_p = 16;

	if(argc < 2) usage();
	imageFileName = argv[1];

	if((blk_dev.fh = fopen(imageFileName, "r")) == NULL)
	{
		printf("Error, cannot open file %s\n", imageFileName);
		return(-1);
	}

	/* Output mtools global variables */
	printf("MTOOLS_NO_VFAT=1\n");
	printf("MTOOLS_SKIP_CHECK=1\n");
	printf("MTOOLS_LOWER_CASE=1\n\n");

	result = mtool_atari_partition (&hd, &blk_dev, 0, 1);
	if ( result == 0 ) {
		doSwap = 1;
		result = mtool_atari_partition (&hd, &blk_dev, 0, 1);
	}

	fclose(blk_dev.fh);
	return(0);
}

void usage()
{

	printf("Usage: mtoolsrcgen filename\nWhere filename is the name of the atari disk image file.\n");
	exit(0);
}
